@@ -138,7 +138,7 @@ sub new {
 	} elsif ($dst) {
 	    # with UDP we can create a connected socket if dst is given
 	    $sockargs{PeerAddr} = $dst->{addr};
-	    $sockargs{PeerPort} = $dst->{port};
+	    $sockargs{PeerPort} = $dst->{port} ||= $default_port;
 	    $sockpeer = $dst;
 	}
 
@@ -165,9 +165,10 @@ sub new {
 	    $self->{src} = ip_sockaddr2parts($saddr);
 	    $self->{src}{host} = $host if $host;
 	}
-	if (!$dst and my $saddr = getpeername($sock)) {
-	    # set dst from connected socket
-	    $sockpeer = $dst = ip_sockaddr2parts($saddr);
+	if ((!$dst or !$sockpeer) and my $saddr = getpeername($sock)) {
+	    # set from connected socket
+	    $sockpeer = ip_sockaddr2parts($saddr);
+	    $dst ||= $sockpeer;
 	}
     }
 
@@ -253,14 +254,10 @@ sub forward_incoming {
 	$uri = $1 if $uri =~m{^<(.*)>};
 	($uri) = sip_hdrval2parts( route => $uri );
 	my $remove_route;
-	if ( $uri eq $self->{contact} ) {
+	my @route = $packet->get_header('route');
+	if (@route and $uri eq $self->{contact}) {
 	    # last router placed myself into URI -> strict router
 	    # get original URI back from last Route-header
-	    my @route = $packet->get_header( 'route' );
-	    if ( !@route ) {
-		# ooops, no route headers? -> DROP
-		return [ '','request from strict router contained no route headers' ];
-	    }
 	    $remove_route = $#route;
 	    $uri = $route[-1];
 	    $uri = $1 if $uri =~m{^<(.*)>};
@@ -268,7 +265,6 @@ sub forward_incoming {
 
 	} else {
 	    # last router was loose,remove top route if it is myself
-	    my @route = $packet->get_header( 'route' );
 	    if ( @route ) {
 		my $route = $route[0];
 		$route = $1 if $route =~m{^<(.*)>};
@@ -410,6 +406,7 @@ sub deliver {
 	my $contact = ( $user =~m{([^<>\@\s]+)\@} ? $1 : $user ).
 	    "\@$addr";
 	$contact = $proto.':'.$contact if $contact !~m{^\w+:};
+	$contact = "<$contact>" if $contact =~m{;};
 	$packet->insert_header( contact => $contact );
     }
     if ( $need_allow && ! ( my @a = $packet->get_header( 'allow' ))) {
